"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const readability_node_1 = require("readability-node");
const xmldom_silent_1 = require("xmldom-silent");
const url_parse_1 = __importDefault(require("url-parse"));
const sanitize_html_1 = __importDefault(require("sanitize-html"));
const clean_html_css_1 = require("./clean-html-css");
const fetch_1 = require("./fetch");
function convertHtmlToXhtml(html) {
    try {
        const xmlSerializer = new xmldom_silent_1.XMLSerializer();
        const xhtmlDocument = new xmldom_silent_1.DOMParser({
            errorHandler: function (level, msg) {
                if (level === "error") {
                    throw new Error(`Unable to convert HTML to XHTML: ${msg}`);
                }
            }
        }).parseFromString(html, "text/html");
        return xmlSerializer.serializeToString(xhtmlDocument);
    }
    catch (e) {
        console.error(e);
    }
}
function createJsDomDocument(xhtml) {
    try {
        const jsDomParser = new readability_node_1.JSDOMParser();
        jsDomParser.parse(xhtml.trim());
        if (jsDomParser.errorState) {
            throw new Error(`Unable to parse XHTML into JsDom ${jsDomParser.errorState}`);
        }
        return jsDomParser.doc;
    }
    catch (e) {
        console.error(e);
    }
}
function createReadabilityUrl(sourceUrl) {
    const sourceUrlParsed = new url_parse_1.default(sourceUrl);
    if (!sourceUrlParsed || sourceUrlParsed.host.length === 0) {
        throw new Error("Invalid or no source url provided");
    }
    return {
        spec: sourceUrlParsed.href,
        host: sourceUrlParsed.host,
        scheme: sourceUrlParsed.protocol.slice(0, -1),
        prePath: `${sourceUrlParsed.protocol}//${sourceUrlParsed.host}`,
        pathBase: `${sourceUrlParsed.protocol}//${sourceUrlParsed.host}${sourceUrlParsed.pathname.substring(0, sourceUrlParsed.pathname.lastIndexOf("/") + 1)}`
    };
}
async function cleanHtml(html, sourceUrl, config = { allowedTags: [], nonTextTags: [] }) {
    try {
        const dataSource = !html && sourceUrl ? await fetch_1.fetchHtml(sourceUrl) : html;
        html = await sanitize_html_1.default(dataSource, {
            allowedTags: [
                ...clean_html_css_1.allowedTags,
                ...((config === null || config === void 0 ? void 0 : config.allowedTags) ? config === null || config === void 0 ? void 0 : config.allowedTags : [])
            ],
            nonTextTags: [
                ...clean_html_css_1.nonTextTags,
                ...((config === null || config === void 0 ? void 0 : config.nonTextTags) ? config === null || config === void 0 ? void 0 : config.nonTextTags : [])
            ]
        });
    }
    catch (e) {
        console.error(e);
    }
    return new Promise(resolve => {
        if (!html) {
            throw new Error("Invalid url or no html provided, please use a html string or url");
        }
        try {
            const readabilityUrl = createReadabilityUrl(sourceUrl);
            const xhtml = convertHtmlToXhtml(html);
            const document = createJsDomDocument(xhtml);
            resolve(new readability_node_1.Readability(readabilityUrl, document).parse());
        }
        catch (error) {
            throw new Error("Unable to clean HTML an issue occured");
        }
    });
}
exports.default = cleanHtml;
//# sourceMappingURL=clean-html.js.map